proc datasets lib = work kill; run;
option linesize = 200 pagesize = 5000 mprint;
dm 'log; clear; output; clear;';

/* setting path */
%let execpath = " ";
%let Path = " ";
%macro setexecpath;
  %let execpath = %sysfunc(getoption(sysin));
  %if %length(&execpath) = 0 %then
    %let execpath = %sysget(sas_execfilepath);
  data _null_;
    do i = length("&execpath") to 1 by -1;
      if substr("&execpath", i, 1) = "\" then do;
        call symput("Path", substr("&execpath", 1, i));
        stop;
      end;
    end;
  run;
%mend setexecpath;
%setexecpath;
libname Out "&Path";

/* set global macro variables for censor extension */
%global color1 color2 color3 color4 scolor1 scolor2 scolor3 scolor4;
%include "&Path.kmdata_v220.sas";
%let filetype = bmp;

data D1;
  length GroupC GroupC2 $20.;
  call streaminit(32789238);
  tc = 12;  /* withdrawal time */
  h1 = 0.100;  n1 = 50; /* h1: hazard of group 1 */
  h2 = 0.075;  n2 = 50; /* h2: hazard of group 2 */
  h3 = 0.050;  n3 = 50; /* h3: hazard of group 3 */
  /* E(T)=1/h */
  array h[*] h1-h3; array n[*] n1-n3;
  do Group = 1 to 3;
    do i = 1 to n[group];
      T = rand('exponential') / h[group]; Censor = 0;
      if T > tc then do; T = tc; Censor = 1; end;
      if rand('Uniform') > 0.9 then do; Censor = 1; end;
/* lost to follow-up (random) */
      if Group = 1 then do; GroupC = '1: high-risk'; GroupC2 = 'high-risk'; end;
      if Group = 2 then do; GroupC = '2: middle-risk'; GroupC2 = 'middle-risk'; end;
      if Group = 3 then do; GroupC = '3: low-risk'; GroupC2 = 'low-risk'; end;
     output;
    end;
  end;
  label T = "Months after entry";
  keep T Censor Group GroupC GroupC2;
run;



/*****************************************************************/
/* Color pattern */
%let color1  = cx445694;
%let color2  = cxA23A2E;
%let color3  = cx01665E;

%let scolor1 = cxD4D9E8;
%let scolor2 = cxF1CECE;
%let scolor3 = cxD1E4E3;



/*****************************************************************/
/* CI: fill, Censor: needle, Atrisk: show, Logrank test, Hazardratio */
%km_data(
  D1, T, GroupC, Censor, 1,
  out = graph, anno = anno,

  CI = 1,
  censEXT = 1,

  Size = 2, 
  atrisk = 1, atriskorder = 0 to 12.5 by 2.5, Step = 5, 
  Label = "No. at risk (1st entry: high, 2nd: middle, 3rd: low)",
  Test = 1, TestX = 98, TestY = 97, Type = logrank,
  HR = 1, HRX = 98, HRY = 92
);
goptions reset = all;
goptions vsize = 12 in hsize = 19 in htitle = 3.5 htext = 3.5;
options linesize = 98 pagesize = 200;
filename grafout "&Path.example1.&filetype.";
goptions device = &filetype. gsfname = grafout gsfmode = replace;
goptions ftext = "Times New Roman";
proc gplot data = Graph;
  plot (Sv1 Sv2 Sv3) * T /
      anno = anno /* autovref cautovref = cxE9DECA */
      overlay skipmiss noframe legend = legend1
      haxis = axis1 vaxis = axis2;
  axis1 label = ('Months after entry') major=(w=2 height=0.7) w=2 minor = none
    order = (0 to 12.5 by 2.5) offset = (2, 7);
  axis2 label = (a=90 'Proportion of overall survival') major=(w=2 height=1)
    w=2 minor = none offset = (2, 2) order = (0 to 1 by 0.2);
  legend1 label = none position = (inside) mode = share across = 2
    origin = (2, 1) value = (h = 2 "high-risk" "middle-risk" "low-risk");
  symbol1 i = steplj c="&color1." w = 5;
  symbol2 i = steplj c="&color2." w = 5;
  symbol3 i = steplj c="&color3." w = 5;
run; quit;



/*****************************************************************/
/* CI: fill, Censor: dot, Atrisk: none, Likelihood ratio test, Hazardratio */
%km_data(
  D1, T, GroupC, Censor, 1,
  out = graph, anno = anno,

  CI = 1,
  censEXT = 0,

  Size = 2, Step = 5,
  atrisk = 0,
  Test = 1,  TestX = 98, TestY = 97, Type = likelihoodratio,
  HR = 1, HRX = 98, HRY = 92
);
goptions reset = all;
goptions vsize = 12 in hsize = 19 in htitle = 3.5 htext = 3.5;
options linesize = 98 pagesize = 200;
filename grafout "&Path.example2.&filetype.";
goptions device = &filetype. gsfname = grafout gsfmode = replace;
goptions ftext = "Times New Roman";
data Graph;
  length vname $10.;
  set Graph;
  var=Sv1; vname='Sv1'; output;
  var=Sv2; vname='Sv2'; output;
  var=Sv3; vname='Sv3'; output;
  var=CSDF1; vname='zCSDF1'; output;
  var=CSDF2; vname='zCSDF2'; output;
  var=CSDF3; vname='zCSDF3'; output;
proc sort data = Graph; by vname T;
proc gplot data = Graph;
  plot var * T = vname/
      anno = anno /* autovref cautovref = cxE9DECA */
      skipmiss noframe legend = legend1
      haxis = axis1 vaxis = axis2;
  axis1 label = ('Months after entry') major=(w=2 height=0.7) w=2 minor = none
    order = (0 to 12.5 by 2.5) offset = (2, 7);
  axis2 label = (a=90 'Proportion of overall survival') major=(w=2 height=1)
    w=2 minor = none offset = (2, 2) order = (0 to 1 by 0.2);
  legend1 label = none position = (inside) mode = share across = 2
    origin = (2, 1) value = (h = 2 "high-risk" "middle-risk" "low-risk")
    order = ("Sv1" "Sv2" "Sv3");
  symbol1 i = steplj c="&color1." w = 5;
  symbol2 i = steplj c="&color2." w = 5;
  symbol3 i = steplj c="&color3." w = 5;
  symbol4 i = none c="&color1." h = 2 v = dot;
  symbol5 i = none c="&color2." h = 2 v = dot;
  symbol6 i = none c="&color3." h = 2 v = dot;
run; quit;



/*****************************************************************/
/* CI: line, Censor: needle, Atrisk: show, MST */
%km_data(
  D1, T, GroupC, Censor, 1,
  out = graph, anno = anno,

  CI = 0,
  censEXT = 1,

  Size = 2, Step = 5, 
  atrisk = 1, atriskorder = 0 to 12.5 by 2.5, 
  Label = "No. at risk (1st entry: high, 2nd: middle, 3rd: low)",
  Test = 0,
  HR = 0,
  MST = 1, MlabX = 65, MmedX = 85, MciX = 98, MSTY = 97
);
goptions reset = all;
goptions vsize = 12 in hsize = 19 in htitle = 3.5 htext = 3.5;
options linesize = 98 pagesize = 200;
filename grafout "&Path.example3.&filetype.";
goptions device = &filetype. gsfname = grafout gsfmode = replace;
goptions ftext = "Times New Roman";
data Graph;
  length vname $10.;
  set Graph;
  var=Sv1; vname='Sv1'; output;
  var=Sv2; vname='Sv2'; output;
  var=Sv3; vname='Sv3'; output;
  var=SL1; vname='zSL1'; output;
  var=SL2; vname='zSL2'; output;
  var=SL3; vname='zSL3'; output;
  var=SU1; vname='zSU1'; output;
  var=SU2; vname='zSU2'; output;
  var=SU3; vname='zSU3'; output;
proc sort data = Graph; by vname T;
proc gplot data = Graph;
  plot var * T = vname/
      anno = anno /* autovref cautovref = cxE9DECA */
      skipmiss noframe legend = legend1
      haxis = axis1 vaxis = axis2;
  axis1 label = ('Months after entry') major=(w=2 height=0.7) w=2 minor = none
    order = (0 to 12.5 by 2.5) offset = (2, 7);
  axis2 label = (a=90 'Proportion of overall survival') major=(w=2 height=1)
    w=2 minor = none offset = (2, 2) order = (0 to 1 by 0.2);
  legend1 label = none position = (inside) mode = share across = 2
    origin = (2, 1) value = (h = 2 "high-risk" "middle-risk" "low-risk")
    order = ("Sv1" "Sv2" "Sv3");
  symbol1 i = steplj c="&color1." w = 5;
  symbol2 i = steplj c="&color2." w = 5;
  symbol3 i = steplj c="&color3." w = 5;
  symbol4 i = steplj c="&scolor1." w = 5 l = 2;
  symbol5 i = steplj c="&scolor2." w = 5 l = 2;
  symbol6 i = steplj c="&scolor3." w = 5 l = 2;
  symbol7 i = steplj c="&scolor1." w = 5 l = 2;
  symbol8 i = steplj c="&scolor2." w = 5 l = 2;
  symbol9 i = steplj c="&scolor3." w = 5 l = 2;
run; quit;



/*****************************************************************/
/* CI: fill, Censor: needle, Atrisk: show, generalized wilcoxon test */
/* Font change!! */
%km_data(
  D1, T, GroupC, Censor, 1,
  out = graph, anno = anno,

  CI = 1,
  censEXT = 1,

  Size = 2, Step = 5, 
  afont = "'Arial'",
  atrisk = 1, atriskorder = 0 to 12.5 by 2.5, 
  Label = "No. at risk (1st entry: high, 2nd: middle, 3rd: low)",
  Test = 1, TestX = 98, TestY = 97, Type = wilcoxon,
  HR = 0
);
goptions reset = all;
goptions vsize = 12 in hsize = 19 in htitle = 3.5 htext = 3.5;
options linesize = 98 pagesize = 200;
filename grafout "&Path.example4.&filetype.";
goptions device = &filetype. gsfname = grafout gsfmode = replace;
goptions ftext = "Arial";
proc gplot data = Graph;
  plot (Sv1 Sv2 Sv3) * T /
      anno = anno /* autovref cautovref = cxE9DECA */
      overlay skipmiss noframe legend = legend1
      haxis = axis1 vaxis = axis2;
  axis1 label = ('Months after entry') major=(w=2 height=0.7) w=2 minor = none
    order = (0 to 12.5 by 2.5) offset = (2, 7);
  axis2 label = (a=90 'Proportion of overall survival') major=(w=2 height=1)
    w=2 minor = none offset = (2, 2) order = (0 to 1 by 0.2);
  legend1 label = none position = (inside) mode = share across = 2
    origin = (2, 1) value = (h = 2 "high-risk" "middle-risk" "low-risk");
  symbol1 i = steplj c="&color1." w = 5;
  symbol2 i = steplj c="&color2." w = 5;
  symbol3 i = steplj c="&color3." w = 5;
run; quit;



/*****************************************************************/
/* CI: line, Censor: needle, Atrisk: show */
/* aspect ratio "12:12" (for presentation), 2group */
data D2; set D1; where Group in (1, 2);
%km_data(
  D2, T, GroupC, Censor, 1,
  out = graph, anno = anno,

  CI = 0,
  censEXT = 1,

  Size = 2, Step = 5, 
  afont = "'Arial'",
  atrisk = 1, atriskorder = 0 to 12.5 by 2.5, 
  Label = "No. at risk (1st entry: high, 2nd: middle)",
  Test = 1,  TestX = 98, TestY = 97, Type = logrank,
  HR = 0
);
goptions reset = all;
goptions vsize = 12 in hsize = 12 in htitle = 3.5 htext = 3.5;
options linesize = 98 pagesize = 200;
filename grafout "&Path.example5.&filetype.";
goptions device = &filetype. gsfname = grafout gsfmode = replace;
goptions ftext = "Arial";
data Graph;
  length vname $10.;
  set Graph;
  var=Sv1; vname='Sv1'; output;
  var=Sv2; vname='Sv2'; output;
  var=SL1; vname='zSL1'; output;
  var=SL2; vname='zSL2'; output;
  var=SU1; vname='zSU1'; output;
  var=SU2; vname='zSU2'; output;
proc sort data = Graph; by vname T;
proc gplot data = Graph;
  plot var * T = vname/
      anno = anno /* autovref cautovref = cxE9DECA */
      skipmiss noframe legend = legend1
      haxis = axis1 vaxis = axis2;
  axis1 label = ('Months after entry') major=(w=2 height=0.7) w=2 minor = none
    order = (0 to 12.5 by 2.5) offset = (2, 7);
  axis2 label = (a=90 'Proportion of overall survival') major=(w=2 height=1)
    w=2 minor = none offset = (2, 2) order = (0 to 1 by 0.2);
  legend1 label = none position = (inside) mode = share across = 1
    origin = (2, 1) value = (h = 2 "high-risk" "middle-risk")
    order = ("Sv1" "Sv2");
  symbol1 i = steplj c="&color1." w = 5;
  symbol2 i = steplj c="&color2." w = 5;
  symbol3 i = steplj c="&scolor1." w = 5 l = 2;
  symbol4 i = steplj c="&scolor2." w = 5 l = 2;
  symbol5 i = steplj c="&scolor1." w = 5 l = 2;
  symbol6 i = steplj c="&scolor2." w = 5 l = 2;
run; quit;




/*****************************************************************/
/* CI: line, baseline adjustment */
data data;
  length strata $20.;
  call streaminit(8797348795);
  tc = 12;  /* time to termination */
  h1 = 0.15; n1 = 100;
  h2 = 0.07; n2 = 100;
  array h[*] h1-h2; array n[*] n1-n2;
  do group = 1 to 2;
    sex = rand("BERNOULLI", 0.1*group);
    baseline = group + rand("Normal");
    do i = 1 to n[group];
      time = baseline + sex + rand('exponential') / h[group]; censor = 0;
      /* termination */
      if time > tc then do; time = tc; censor = 1; end;
      /* random censoring */
      if rand('Uniform') > 0.9 then do; censor = 1; end;
      if group = 1 then do; strata = "1: Group-1"; end;
      if group = 2 then do; strata = "2: Group-2"; end;
     output;
    end;
  end;
  label time = "Months after entry";
  keep time censor group strata;
run;
proc summary data = data;
  class strata;
  var sex;
  output out = Cov00s mean = Sex;
proc summary data = data;
  class strata;
  var baseline;
  output out = Cov00b mean = Baseline;
data COV00;
  merge Cov00s Cov00b;
  by strata;
  where strata = "";
  strata = "1: Group-1"; output;
  strata = "2: Group-2"; output;
  keep strata sex baseline;
run;

%km_data(
data, time, strata, censor, 1, 
out = graph, afont = "'Times New Roman'",
adjbase = 1, adjbase_data = Cov00, covariates = sex baseline
);
goptions reset = all;
goptions vsize = 12 in hsize = 19 in htitle = 3.5 htext = 3.5;
filename grafout "&Path.example6.&filetype.";
goptions device = &filetype. gsfname = grafout gsfmode = replace;
goptions ftext = "Times New Roman";
data Graph;
  length vname $10.;
  set Graph;
  var=Sv1; vname='Sv1'; output;
  var=Sv2; vname='Sv2'; output;
  var=SL1; vname='ySL1'; output;
  var=SL2; vname='ySL2'; output;
  var=SU1; vname='ySU1'; output;
  var=SU2; vname='ySU2'; output;
proc gplot data = graph;
plot var * time = vname /
  autovref cautovref = cxE9DECA
  overlay skipmiss noframe legend = legend1
  haxis = axis1 vaxis = axis2;
axis1 label = ('Months from entry') major=(w=2 height=0.7) w=2 minor = none
  offset = (2, 7);
axis2 label = (a=90 'Overall survival proportion') major=(w=2 height=1)
  w=2 minor = none offset = (2, 2) order = (0 to 1 by 0.2);
legend1 label = none position = (inside)
  mode = share across = 2 origin = (2, 1) value = (h = 2 "1: Group-1" "2: Group-2")
  order = ("Sv1" "Sv2");
symbol1 i = steplj c="&color1" w = 5;
symbol2 i = steplj c="&color2" w = 5;
symbol3 i = steplj c="&scolor1" w = 5 l = 2;
symbol4 i = steplj c="&scolor2" w = 5 l = 2;
symbol5 i = steplj c="&scolor1" w = 5 l = 2;
symbol6 i = steplj c="&scolor2" w = 5 l = 2;
run; quit;




/*****************************************************************/
/* CI: line, Failure plot */
data data;
  length strata $20.;
  tc = 12;  /* time to termination */
  h1 = 0.15; n1 = 100;
  h2 = 0.07; n2 = 100;
  array h[*] h1-h2; array n[*] n1-n2;
  do group = 1 to 2;
    do i = 1 to n[group];
      time = rand('exponential') / h[group]; censor = 0;
      /* termination */
      if time > tc then do; time = tc; censor = 1; end;
      /* random censoring */
      if rand('Uniform') > 0.9 then do; censor = 1; end;
      if group = 1 then do; strata = "1: Group-1"; end;
      if group = 2 then do; strata = "2: Group-2"; end;
     output;
    end;
  end;
  label time = "Months after entry";
  keep time censor group strata;
run;

%km_data(
data, time, strata, censor, 1, 
out = graph, anno = anno, afont = "'Times New Roman'", 
failure = 1, 
censEXT  = 1
);
goptions reset = all;
goptions vsize = 12 in hsize = 19 in htitle = 3.5 htext = 3.5;
filename grafout "&Path.example7.&filetype.";
goptions device = &filetype. gsfname = grafout gsfmode = replace;
goptions ftext = "Times New Roman";
data Graph;
  length vname $10.;
  set Graph;
  var=Sv1; vname='Sv1'; output;
  var=Sv2; vname='Sv2'; output;
  var=SL1; vname='ySL1'; output;
  var=SL2; vname='ySL2'; output;
  var=SU1; vname='ySU1'; output;
  var=SU2; vname='ySU2'; output;
proc gplot data = graph;
plot var * time = vname /
  anno = anno autovref cautovref = cxE9DECA
  overlay skipmiss noframe legend = legend1
  haxis = axis1 vaxis = axis2;
axis1 label = ('Months from entry') major=(w=2 height=0.7) w=2 minor = none
  offset = (2, 7);
axis2 label = (a=90 'Overall survival proportion') major=(w=2 height=1)
  w=2 minor = none offset = (2, 2) order = (0 to 1 by 0.2);
legend1 label = none position = (inside)
  mode = share across = 2 origin = (2, 1) value = (h = 2 "1: Group-1" "2: Group-2")
  order = ("Sv1" "Sv2");
symbol1 i = steplj c="&color1" w = 5;
symbol2 i = steplj c="&color2" w = 5;
symbol3 i = steplj c="&scolor1" w = 5 l = 2;
symbol4 i = steplj c="&scolor2" w = 5 l = 2;
symbol5 i = steplj c="&scolor1" w = 5 l = 2;
symbol6 i = steplj c="&scolor2" w = 5 l = 2;
run; quit;

